#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::FacetedSearchAttribute
# state        private
# description  FacetedSearchAttribute test.
#========================================================================================
package DE_EPAGES::RemoteSearch::t::FacetedSearchAttribute;
use DE_EPAGES::Core::API::File qw ( RunOnDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DataCache::API::Cache qw ( UpdateLocal );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw ( ImportAttributes );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::RemoteSearch::DAL::FacetedSearch qw ( DAL_GetInfoFacetedSearchAttribute );
use File::Basename;
use Test::More;

use base 'Test::Class';

use strict;

my $ROOTALIAS = 'Test';
my $ROOTPATH = JoinObjectPath(['',$ROOTALIAS]);

sub init : Test(startup) {
    my $self = shift;
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    $self->cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTALIAS } );
    ImportObjects( 'import.xml', $ROOTPATH );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');
    ImportAttributes( 'import_en.xml', $ROOTPATH . '/TestShopRemoteSearch');
    return 1;
}

sub cleanup : Test(shutdown) {
    my $self = shift;
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
}

sub setUpTest : Test(setup) {
    my $self = shift;
    $self->{Shop}  = LoadObjectByPath(JoinObjectPath(['',$ROOTALIAS, 'TestShopRemoteSearch']));
}

sub testNonPredef : Test(no_plan) {
    my $self = shift;
    my $Comment = $self->shop->path('ProductClass/Attributes/Comment');
    is($Comment->get('IsSearchFilter'), 0, 'Comment IsSearchFilter=0');

    ok(!defined(DAL_GetInfoFacetedSearchAttribute($Comment->id)), 'Comment database non-existant.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Comment->id);

    # set without a change.
    $Comment->set({'IsSearchFilter' => 0});
    is($Comment->get('IsSearchFilter'), 0, 'Comment IsSearchFilter=0');
    ok(!defined(DAL_GetInfoFacetedSearchAttribute($Comment->id)), 'Comment database non-existant.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Comment->id);

    # set with a change
    $Comment->set({'IsSearchFilter' => 1});
    is(DAL_GetInfoFacetedSearchAttribute($Comment->id)->{'IsSearchFilter'}, 1, 'Comment database is one.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Comment->id);

    # set with a delete.
    $Comment->set({'IsSearchFilter' => 0});
    ok(!defined(DAL_GetInfoFacetedSearchAttribute($Comment->id)), 'Comment database non-existant.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Comment->id);
}

sub testPreDefAttribute : Test(no_plan) {
    my $self = shift;
    my $Color   = $self->shop->path('ProductTypes/Jacket/Attributes/Color');
    my $Size    = $self->shop->path('ProductTypes/Jacket/Attributes/Size');

    UpdateLocal();
    is($Color->get('IsSearchFilter'), 1, 'Color IsSearchFilter=1');
    is($Size->get('IsSearchFilter'), 1, 'Size IsSearchFilter=1');

    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 1, 'Color database entry is one.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Color->id);

    # set without a change.
    $Color->set({'IsSearchFilter' => 1});
    is($Color->get('IsSearchFilter'), 1, 'Color IsSearchFilter=1 after set.');
    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 1, 'Color database entry is still one.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Color->id);

    # set with an update to zero
    $Color->set({'IsSearchFilter' => 0});
    is($Color->get('IsSearchFilter'), 0, 'Color IsSearchFilter=0 after set.');
    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 0, 'Color database entry is zero.');

    # make sure we don't see effects from the cache
    UpdateLocal();
    is($Color->get('IsSearchFilter'), 0, 'Color IsSearchFilter=0 after cache reset.');
    UpdateLocal();
    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 0, 'Color database entry is zero after cache reset.');

    # set with an update to one.
    $Color->set({'IsSearchFilter' => 1});
    is($Color->get('IsSearchFilter'), 1, 'Color IsSearchFilter=1 after second set.');
    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 1, 'Color database entry is one again.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Color->id);
    UpdateLocal();
    is($Color->get('IsSearchFilter'), 1, 'Color IsSearchFilter=1 after cache reset.');
    UpdateLocal();
    is(DAL_GetInfoFacetedSearchAttribute($Color->id)->{'IsSearchFilter'}, 1, 'Color database entry is one after cache reset.')
      or diag explain DAL_GetInfoFacetedSearchAttribute($Color->id);
}

sub shop {
    return shift->{Shop};
}

####### Main ############
RunScript(
    Sub => sub {
        RunOnDirectory(
            dirname(__FILE__),
            sub {
                RunOnStore(
                    'Store' => GetTestSite(),
                    'Sub'   => sub {
                        Test::Class->runtests;
                    }
                );
            }
        );
    }
);

1;